//-------------------------------------------------------------------------
//
//  Main Javascript file for richweather mobile app
//  Finds location of the device and loads relevant weather from the server
//  Uses Intel XDK/Intel App Framework with af.carousel.js plugin
//  
//  Author:         Ben Jacobs
//  Date modified:  17/08/2014
//
//-------------------------------------------------------------------------

var toggles = ["ON","ON","ON","ON","ON","ON","ON","ON"];
//names for saving as cookie
var toggle_names = ['weather_toggle','need_toggle','wear_toggle','wind_toggle',
                    'washing_toggle','hair_toggle','pollen_toggle','shoes_toggle'];
var page_titles = ['the weather will be...', "you'll need...",
                    'you should wear...', 'the wind will be...',
                    'you should dry your washing...', 'hair frizz potential is...',
                    'pollen sneeze count is...', 'your shoes should be...'];
//<img> ids for setting images
var image_ids = ['weather_image', 'need_image', 'wear_image', 'wind_image',
                 'washing_image', 'hair_image', 'pollen_image', 'shoes_image'];
var NUM_PAGES = 8; //max. number of pages
var url_base = 'http://www.richweather.co.uk/locations/current_weather/';
var url_location;
var current_latitude;
var current_longitude;
var CAROUSEL_INTITIALISED = false;
var DAY_INDEX = 0; //0 today, 1 tomorrow
var previous_page = "";
var LOCATION_ID = 0; // \
var LATITUDE = 1;    //  | enums for accessing
var LONGITUDE = 2;   //  |  locations array
var NAME = 3;        // /
var auto;
var auto_index;

//*****************************************************************************
//   run as soon as Intel activates
//*****************************************************************************
var onDeviceReady=function(){
    intel.xdk.device.hideSplashScreen();
    try {
       intel.xdk.device.setRotateOrientation("portrait");
    } catch(err) {
        intel.xdk.notification.alert("Error: " + err.name + " " + err.message,"ERROR (setRotateOrientation)","Okay");
    }
    try {
       intel.xdk.device.setAutoRotate(false);
    } catch(err) {
        intel.xdk.notification.alert("Error: " + err.name + " " + err.message,"ERROR (setAutoRotate)","Okay");
    }
    
    //Scale the display
    try {
        setupScale(480);
        
        // lockViewportWindow doesn't work on some devices (e.g. HTCs) so use setupScale instead
        //intel.xdk.display.lockViewportWindow(480,640,640,480);
        //var height = intel.xdk.display.window.portheight;
        //if(height == 0) {
        //  //lockViewportWindow didn't work... 
        //}
    } catch(err) {
        intel.xdk.notification.alert("Error: " + err.name + " " + err.message,"ERROR (setupScale)","Okay");
    }
    $("#logo").bind("doubleTap", function() {
        intel.xdk.notification.confirm("Update current location?",
                                       'updateLocation', "Location",
                                       "Yes", "No");
    });
    $("#logo").bind("singleTap", function() {
        goToPage('location_page');
    });
    $("#button_home").bind("doubleTap", function() {
        goToPage('main_page');
        populateCarousel("update"); 
    });
    $("#button_home").bind("singleTap", function() {
        goToPage('main_page');
    });
    
    loadSettings();
    loadSavedWeather();
    getLocation();
    setAutocomplete();    
    showHelp(-1);
};
document.addEventListener("intel.xdk.device.ready",onDeviceReady,false);

//*****************************************************************************
//  prevents users from dragging the page
//*****************************************************************************
var preventDefaultScroll = function(event) {
    window.scroll(0,0);
    event.preventDefault();
    return false;
};
document.addEventListener('touchmove', preventDefaultScroll, false);

//  Set up confirmation event listener
document.addEventListener('intel.xdk.notification.confirm', receiveConfirm, false);
//Process the event for the confirmed message
function receiveConfirm(e){
    if( e.id == 'updateLocation'){
        if( e.success == true && e.answer == true ) {
            getLocation();
        }
    }
}
//*****************************************************************************
// Set weather location from user input
//*****************************************************************************
function manualLocation() {
    document.getElementById('search_status').innerHTML = "Searching...";
    
    var name = auto.getText();
    var found = 0;
    for(var i=0; i < locations[LOCATION_ID].length; i++) {
        if(name == locations[NAME][i]) {
            intel.xdk.cache.setCookie("location_id", locations[LOCATION_ID][i] , -1);
            intel.xdk.cache.setCookie("location_name", locations[NAME][i] , -1);
            
            url_location = locations[LOCATION_ID][i] + '.xml';
            document.getElementById('location_header').innerHTML = locations[NAME][i];
            
            populateCarousel("update");
            found = 1;
            document.getElementById('search_status').innerHTML = "Location updated";
            break;
        }
    }
    if(found == 0) {
        document.getElementById('search_status').innerHTML = "Oops, I can't find that location";
    }
}
//*****************************************************************************
//  if currrent position is avaible, use locations list to find best fit
//*****************************************************************************
function getLocation() {
    document.getElementById('location_header').innerHTML += " (updating)"; 
    
    //intel.xdk.notification.alert("getLocation","DEBUG MESSAGE","Okay");
    var success = function(p){
        //intel.xdk.notification.alert("getCurrentPosition success","DEBUG MESSAGE","Okay");
        if (p.coords.latitude != undefined){
            current_latitude = p.coords.latitude;
            current_longitude = p.coords.longitude;
            
            //intel.xdk.notification.alert("latitude: " + current_latitude + " longitude: " + current_longitude,"DEBUG MESSAGE","Okay"); 
            
            //  loop through locations, find best location based on min distance
            //  run the function to populate carousel when location found
            var best_location = [];
            best_location['distance'] = 100000;
            best_location['id'] = "";
            best_location['name'] = "";
            
            for(var i=0;i< locations[LOCATION_ID].length;i++) {
                var distance = getDistance( parseFloat(locations[LONGITUDE][i]),
                                            parseFloat(locations[LATITUDE][i])    );
                if(distance < best_location['distance']) {
                    best_location['distance'] = distance;
                    best_location['id'] = locations[LOCATION_ID][i];
                    best_location['name'] = locations[NAME][i];
                }
            }
            
            intel.xdk.cache.setCookie("location_id", best_location['id'] , -1);
            intel.xdk.cache.setCookie("location_name", best_location['name'] , -1);
            
            url_location = best_location['id'] + '.xml';
            document.getElementById('location_header').innerHTML = best_location['name']; 
            
            populateCarousel("update");
        } else {
            intel.xdk.notification.alert("coords undefined","DEBUG MESSAGE","Okay"); 
        }
    };
    var error = function(){
        var id = intel.xdk.cache.getCookie("location_id");
        var name = intel.xdk.cache.getCookie("location_name");
        if(id != "undefined" && name != "undefined") {
            intel.xdk.notification.alert("Using last known location","Location Unavailable","Okay");        
            url_location = id + '.xml';
            document.getElementById('location_header').innerHTML = name;   
            
            populateCarousel("update"); 
        } else {
            document.getElementById('carousel').innerHTML = 'unable to access your location'; 
        }
    };
    try {
        intel.xdk.geolocation.getCurrentPosition(success,error);
    } catch(err) {
        intel.xdk.notification.alert("Error: " + err.name + " " + err.message,"ERROR (getCurrentPosition)","Okay");
    }
}
//*****************************************************************************
//  returns distance between given points and current location
//*****************************************************************************
function getDistance(longitude, latitude) {  
    var R = 6371; // km
    //convert to radians
    var dLat = (current_latitude-latitude)* Math.PI / 180;
    var dLon = (current_longitude-longitude)* Math.PI / 180;
    var lat1 = (latitude)* Math.PI / 180;
    var lat2 = (latitude)* Math.PI / 180;
    
    var a = Math.sin(dLat/2) * Math.sin(dLat/2) +
        Math.sin(dLon/2) * Math.sin(dLon/2) * Math.cos(lat1) * Math.cos(lat2); 
    return R * 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
}

//***************************************************************************
//  build up pages and initialise/update carousel div, populate pages
//*****************************************************************************
function populateCarousel(action) {
    var carousel_content = "";
    var count = 0;
    for(var i=0; i < NUM_PAGES; i++) { 
        if(toggles[i] == "ON") {
            carousel_content += '<div class="page_'+ count +'">';
            carousel_content += '<div class="page_title">';
            carousel_content += page_titles[i];
            carousel_content += '</div>';
            carousel_content += '<img class="page_image ' + image_ids[i] + '" src="" />';
            carousel_content += '</div>';
            count++;
        }
    }
    if(CAROUSEL_INTITIALISED == false) {
        document.getElementById('carousel').innerHTML = carousel_content;
        $("#carousel").carousel({
            vertical:false, // page up/down
            horizontal:true, // page left/right
            pagingDiv:"carousel_dots", // div to hold the dots for paging
            pagingCssName:"carousel_paging", //classname for the paging dots
            pagingCssNameSelected: "carousel_paging_selected", //classname for the selected page dots
            wrap:true //Creates a continuous carousel
        });
        CAROUSEL_INTITIALISED = true;
    } else if(CAROUSEL_INTITIALISED == true) {
        document.getElementById('carousel').children[0].innerHTML = carousel_content;
        $("#carousel").carousel().refreshItems();
        $("#carousel").carousel().onMoveIndex(0, 0);
    }
    if(action == "update") {
        try {
            intel.xdk.device.getRemoteData(url_base + url_location,"GET","","updateWeather","errorLoading");
        } catch(err) {
            intel.xdk.notification.alert("Error: " + err.name + " " + err.message,"ERROR (getRemoteData)","Okay");
        }
    }
}

//*****************************************************************************
//  get relevant information from xml data (image names)
//*****************************************************************************
function updateWeather(data) {
    checkTime();
    var current_weather = $.parseXML(data);

    var day = $(current_weather).find("day").eq(DAY_INDEX).text();
    document.getElementById("day_of_the_week").innerHTML= day;
    setImages([ $(current_weather).find("weather").eq(DAY_INDEX).text(),
                $(current_weather).find("need").eq(DAY_INDEX).text(),
                $(current_weather).find("wear").eq(DAY_INDEX).text(),
                $(current_weather).find("wind").eq(DAY_INDEX).text(),
                $(current_weather).find("washing").eq(DAY_INDEX).text(),                
                $(current_weather).find("hair").eq(DAY_INDEX).text(),
                $(current_weather).find("pollen").eq(DAY_INDEX).text(),
                $(current_weather).find("shoes").eq(DAY_INDEX).text()     ]);
    
    saveWeather([ $(current_weather).find("weather").eq(DAY_INDEX).text(),
                  $(current_weather).find("need").eq(DAY_INDEX).text(),
                  $(current_weather).find("wear").eq(DAY_INDEX).text(),
                  $(current_weather).find("wind").eq(DAY_INDEX).text(),
                  $(current_weather).find("washing").eq(DAY_INDEX).text(),                
                  $(current_weather).find("hair").eq(DAY_INDEX).text(),
                  $(current_weather).find("pollen").eq(DAY_INDEX).text(),
                  $(current_weather).find("shoes").eq(DAY_INDEX).text(),
                  $(current_weather).find("day").eq(DAY_INDEX).text()     ]);
}

//*****************************************************************************
//  set all pages to misc_error image
//*****************************************************************************
function errorLoading(data) {
    setImages("misc_error");   
}

//*****************************************************************************
//  set images based on image_ids array
//*****************************************************************************
function setImages(image) {
    if(image.length == 1) { //same for each page
        for(var i=0; i < NUM_PAGES; i++) { 
            if(toggles[i] == "ON") {
                //the carousel duplicates the first and last pages
                // need to use a class and set source image of all instances
                var elements = document.getElementsByClassName(image_ids[i]);
                for (var j = 0; j < elements.length; j++) {
                    elements[j].src= "images/" + image + ".png";
                }
            }
        }
    } else if(image.length == 8){
        for(var i=0; i < NUM_PAGES; i++) { 
            if(toggles[i] == "ON") {
                var elements = document.getElementsByClassName(image_ids[i]);
                for (var j = 0; j < elements.length; j++) {
                    elements[j].src= "images/" + image[i] + ".png";
                }
            }
        }
    }else {
        //error
    } 
}
//*****************************************************************************
//  Change to the given page
//*****************************************************************************
function goToPage(destination_page) {
    var current = getCurrentPage();
    if(current == "settings_page" && destination_page == "settings_page") {
        destination_page = "main_page";
    } else if(current == "location_page" && destination_page == "location_page") {
        destination_page = "main_page";
    } else if(current == "main_page" && destination_page == "main_page") {
        $("#carousel").carousel().onMoveIndex(0, 1000);
    }
    if(current != destination_page) {
        if(current == "settings_page") {
            saveSettings();
            loadSavedWeather(); 
        }
        
        document.getElementById("main_page").style.display="none";
        document.getElementById("settings_page").style.display="none";
        document.getElementById("location_page").style.display="none";
        
        document.getElementById(destination_page).style.display="block";
    }
}
//*****************************************************************************
//  Get current page
//*****************************************************************************
function getCurrentPage() {
    if(document.getElementById("main_page").style.display=="block") {
        return "main_page";
    } else if(document.getElementById("settings_page").style.display=="block") {
        return "settings_page";
    } else if(document.getElementById("location_page").style.display=="block") {
        return "location_page";
    }
}
//*****************************************************************************
//  Load and display saved weather
//*****************************************************************************
function loadSavedWeather() {
    var images = [];
    var day = "";
    var location = "";
    var check = false;
    for(var i=0; i < NUM_PAGES; i++) {            
        if(intel.xdk.cache.getCookie(image_ids[i]) != "undefined") {
            images[i] = intel.xdk.cache.getCookie(image_ids[i]);
            check = true;
        } else {
            images[i] = "misc_error";
        }
    }
    if(intel.xdk.cache.getCookie("day_of_the_week") != "undefined") {
        day = intel.xdk.cache.getCookie("day_of_the_week");
    }
    if(intel.xdk.cache.getCookie("location_name") != "undefined") {
        location = intel.xdk.cache.getCookie("location_name");
    }
    if(check === true) {
        populateCarousel("no_update");
        setImages(images);
        document.getElementById("day_of_the_week").innerHTML = day;
        document.getElementById('location_header').innerHTML = location;
    }
}
//*****************************************************************************
//  save weather as cookies
//*****************************************************************************
function saveWeather(weather_values) {
    if(weather_values.length !== 0) {
        for(var i=0; i < NUM_PAGES; i++) {  
            intel.xdk.cache.setCookie(image_ids[i], weather_values[i] , -1);
        }
        //last is day of the week
        intel.xdk.cache.setCookie("day_of_the_week", weather_values[NUM_PAGES] , -1);
    }
}
//*****************************************************************************
//  set the toggle array as appropriate from the saved cookies
//*****************************************************************************
function loadSettings() {
    if(intel.xdk.cache.getCookieList().length !== 0) {
        for(var i=0; i < NUM_PAGES; i++) {            
            if(intel.xdk.cache.getCookie(toggle_names[i]) == "OFF") {
                document.getElementById(toggle_names[i]).checked = false;
                toggles[i] = "OFF";
            }
        }
    }
}
//*****************************************************************************
//  set the toggle array from the checkboxes, and save/update the cookies
//*****************************************************************************
function saveSettings() {
    for(var i=0; i < NUM_PAGES; i++) {
        if(document.getElementById(toggle_names[i]).checked) {
            toggles[i] = "ON";
        } else {
            toggles[i] = "OFF";   
        }
        intel.xdk.cache.setCookie(toggle_names[i], toggles[i] , -1);
    }
}
//*****************************************************************************
//  Show help when first using the app
//*****************************************************************************
function showHelp(number) {       
    document.getElementById("help").style.display = "block";

    document.getElementById("help_1").style.display = "none";
    document.getElementById("help_2").style.display = "none";
    document.getElementById("help_3").style.display = "none";
    document.getElementById("help_4").style.display = "none";
    
    switch(number) {
        case -1:
            if(intel.xdk.cache.getCookie("help") != "hide_help") {
                showHelp(0);
            }
            break;
        case 0:
            document.getElementById("main_page").style.display="block";
            document.getElementById("settings_page").style.display="none";
            document.getElementById("location_page").style.display="none"; 
            
            document.getElementById("help_1").style.display = "block";
            break;
        case 1:            
            document.getElementById("main_page").style.display="none";
            document.getElementById("settings_page").style.display="none";
            document.getElementById("location_page").style.display="block"; 
            
            document.getElementById("help_2").style.display = "block";
            break;
        case 2:
            document.getElementById("help_2").style.display = "none";            
            document.getElementById("help_3").style.display = "block";
            break;
        case 3:
            document.getElementById("main_page").style.display="none";
            document.getElementById("settings_page").style.display="block";
            document.getElementById("location_page").style.display="none"; 
            
            document.getElementById("help_4").style.display = "block";
            break;
        case 4:
            document.getElementById("main_page").style.display="block";
            document.getElementById("settings_page").style.display="none";
            document.getElementById("location_page").style.display="none"; 
            intel.xdk.cache.setCookie("help", "hide_help" , -1);
            break;
    }
}
//*****************************************************************************
//  Autocomplete function for location suggestions when manually searching
//*****************************************************************************
function setAutocomplete() {
    auto = completely(document.getElementById('manual_location'), {
    	fontSize : '22px',
    	fontFamily :'Arial Rounded MT Bold',
        color: '#21007F'
    });
    auto.dropDown.style.maxWidth = '100%';
    auto.dropDown.style.top = '-5em';
    auto.dropDown.style.height = '5em';
    auto.onChange = function(txt) {
        switch(txt.length) {
            case 0:
                auto.options = [''];
                break;
            case 1:
                var index = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'.indexOf(txt);
                if(index > 25) {
                    index = index - 26;
                }
                auto_index = index;
                break;
            case 2:
                auto.setText(txt.capitalise());
                auto.options = location_names[auto_index];
                break;
        }
        auto.repaint();
        if(txt.length > 1) {
            try {
                auto.dropDown.style.top = '-5em';
                var myOtherScroller=auto.dropDown.scroller({
                    verticalScroll:true,
                    horizontalScroll:false,
                    autoEnable:true,
                    lockBounce:true
                })
            } catch (err) {
                
            }
        }
    }   
}
String.prototype.capitalise = function() {
    return this.charAt(0).toUpperCase() + this.slice(1);
}
//if past 21:00 get tomorrow's weather
function checkTime() {
    var date = new Date();
    if(date.getHours() > 20) {
        DAY_INDEX = 1;
    }
}

/*
 * Makes sure that the device is scaled so that it is at least minWidth px in 
 * width in any orientation. This is done by setting the zoom appropriately.
 */
function setupScale (minWidth) {
    var viewWidth = Math.max(document.documentElement.clientWidth, window.innerWidth);
    var viewHeight = Math.max(document.documentElement.clientHeight, window.innerHeight);
    var portWidth = Math.min(viewWidth, viewHeight);
    var landWidth = Math.max(viewWidth, viewHeight);
    var fixScale = function () {
        // portrait      
        if (Math.abs(window.orientation) != 90) {
            document.body.style.zoom = portWidth / minWidth;
        }
        // landscape, but width is less than minWidth so can scale down
        else if (landWidth < minWidth) {
            document.body.style.zoom = landWidth / minWidth;
        } 

        // landscape but width is greater than minWidth so turn off zoom
        //  instead of scaling up and looking bad
        else {
            document.body.style.zoom = 1;
        }
    };

   // device is greater than minWidth even in portrait, also don't scale up
   if (portWidth == minWidth) {
       return;
   } 

   fixScale();                                       // fix the current scale
   window.onorientationchange = fixScale;            // and when orientation is changed
}
